# Gestion des logs avec Flask
![category](développement)
![tag](flask)
![tag](python)

Très vite quand on travaille avec Flask on souhaite savoir ce qui se passe
et donc des logs.

Heureusement le framework comporte un logger natif.
Il suffit d'ajouter ces propres handlers pour obtenir les logs dans le format
souhaité:

- fichier
- graylog
- graylog par rabbit
- ...


```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import logging, logging.handlers
import uuid
import time

from flask import Flask, abort, request
import graypy


LEVEL = {
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
    "WARNING": logging.WARNING,
    "ERROR": logging.ERROR,
    "CRITICAL":logging.CRITICAL,
}

EX_IN = os.environ['EX_IN']
MQ_user = os.environ['MQ_USER']
MQ_password = os.environ['MQ_PASSWORD']
MQ_host = os.environ['MQ_HOST']
LogLevel = LEVEL[os.environ['LOG_LEVEL']]
LogFilename = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'log', os.environ['LOG_FILENAME'])
LogMaxbytes = int(os.environ['LOG_MAXBYTES'])
LogBackcount = int(os.environ['LOG_BACKCOUNT'])
LogGraylogMq = os.environ['LOG_GRAYLOG_MQ']
LogGraylogEx = os.environ['LOG_GRAYLOG_EX']
HOST = os.environ['HOST']
PORT = int(os.environ['PORT'])
PG = "dbname=postgres user=postgres host='postgres' password='postgres'"

app = Flask(__name__)
############################ Route #############################################

@app.route("/hello")
def url_hello():
    app.logger.info('in hello')
    return "hello"
    
if __name__ == "__main__":
    if not app.debug:
        app.logger.setLevel(LogLevel)
        file_handler = logging.handlers.RotatingFileHandler(LogFilename,
                            maxBytes=LogMaxbytes,
                            backupCount=LogBackcount)
        file_handler.setLevel(LogLevel)
        app.logger.addHandler(file_handler) 
        graylog_handler = graypy.GELFRabbitHandler(LogGraylogMq, LogGraylogEx)
        graylog_handler.setLevel(LogLevel)
        app.logger.addHandler(graylog_handler)
    app.run(host = HOST,
            port = PORT,
            threaded=True)
```

Nous récupérons les paramètres via l'environnement du système.

Reste une question comment avoir une trace complète pour tous les routes avec par exemple
le temps de traitement?

Il suffit de créer une fonction qui va décorer toutes nos routes


```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import logging, logging.handlers
import uuid
import time

from flask import Flask, abort, request
import graypy


LEVEL = {
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
    "WARNING": logging.WARNING,
    "ERROR": logging.ERROR,
    "CRITICAL":logging.CRITICAL,
}

EX_IN = os.environ['EX_IN']
MQ_user = os.environ['MQ_USER']
MQ_password = os.environ['MQ_PASSWORD']
MQ_host = os.environ['MQ_HOST']
LogLevel = LEVEL[os.environ['LOG_LEVEL']]
LogFilename = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'log', os.environ['LOG_FILENAME'])
LogMaxbytes = int(os.environ['LOG_MAXBYTES'])
LogBackcount = int(os.environ['LOG_BACKCOUNT'])
LogGraylogMq = os.environ['LOG_GRAYLOG_MQ']
LogGraylogEx = os.environ['LOG_GRAYLOG_EX']
HOST = os.environ['HOST']
PORT = int(os.environ['PORT'])
PG = "dbname=postgres user=postgres host='postgres' password='postgres'"

app = Flask(__name__)

from functools import wraps
from flask import g, request, redirect, url_for

def trace(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        start_time = time.time()  
        res = f(*args, **kwargs)
        interval = time.time() - start_time 
        app.logger.debug('call %s with %s and %s time: %s' % (f.__name__, args, kwargs, interval))
        return res
    return decorated_function


app.trace = trace


############################ Route #############################################

@app.route("/hello")
@app.trace
def url_hello():
    app.logger.info('in hello')
    return "hello"
    
if __name__ == "__main__":
    if not app.debug:
        app.logger.setLevel(LogLevel)
        file_handler = logging.handlers.RotatingFileHandler(LogFilename,
                            maxBytes=LogMaxbytes,
                            backupCount=LogBackcount)
        file_handler.setLevel(LogLevel)
        app.logger.addHandler(file_handler) 
        graylog_handler = graypy.GELFRabbitHandler(LogGraylogMq, LogGraylogEx)
        graylog_handler.setLevel(LogLevel)
        app.logger.addHandler(graylog_handler)
    app.run(host = HOST,
            port = PORT,
            threaded=True)
```
